import os

from truss.templates.control.control.helpers.context_managers import current_directory


def test_current_directory(tmp_path):
    orig_cwd = os.getcwd()
    with current_directory(tmp_path):
        assert os.getcwd() == str(tmp_path)

    assert os.getcwd() == orig_cwd



import inspect
import logging
import sys
from ast import ClassDef, FunctionDef
from dataclasses import dataclass
from typing import Any, Dict, List, Tuple

logger: logging.Logger = logging.getLogger(__name__)


@dataclass
class ModelBuildStageOne:
    # the Python Class of the model
    model_type: str
    # the framework that the model is built in
    model_framework: str


def _model_class(model: Any):
    return model.__class__


def infer_python_version() -> str:
    return f"py{sys.version_info.major}{sys.version_info.minor}"


def map_to_supported_python_version(python_version: str) -> str:
    """Map python version to truss supported python version.

    Currently, it maps any versions greater than 3.9 to 3.9.

    Args:
        python_version: in the form py[major_version][minor_version] e.g. py39,
        py310
    """
    python_major_version = int(python_version[2:3])
    python_minor_version = int(python_version[3:])

    if python_major_version > 3:
        raise NotImplementedError("Only python version 3 is supported")

    if python_minor_version > 11:
        logger.info(
            f"Mapping python version {python_major_version}.{python_minor_version}"
            " to 3.11, the highest version that Truss currently supports."
        )
        return "py311"

    if python_minor_version < 8:
        logger.info(
            f"Mapping python version {python_major_version}.{python_minor_version}"
            " to 3.8, the lowest version that Truss currently supports."
        )
        return "py38"

    return python_version


def _infer_model_init_parameters(model_class: Any) -> Tuple[List, List]:
    full_arg_spec = inspect.getfullargspec(model_class.__init__)
    named_args = full_arg_spec.args[1:]
    number_of_kwargs = full_arg_spec.defaults and len(full_arg_spec.defaults) or 0
    required_args = full_arg_spec.args[1:-number_of_kwargs]
    return named_args, required_args


def _infer_model_init_parameters_ast(model_class_def: ClassDef) -> Tuple[List, List]:
    named_args: List[str] = []
    required_args: List[str] = []
    init_model_functions = [
        node
        for node in model_class_def.body
        if isinstance(node, FunctionDef) and node.name == "__init__"
    ]

    if not init_model_functions:
        return named_args, required_args

    assert (
        len(init_model_functions) == 1
    ), "There should only be one __init__ function in the model class"
    init_model_function = init_model_functions[0]
    named_args = [arg.arg for arg in init_model_function.args.args][1:]
    number_of_defaults = len(init_model_function.args.defaults)
    required_args = named_args[:-number_of_defaults]
    return named_args, required_args


def validate_provided_parameters_with_model(
    model_class: Any, provided_parameters: Dict[str, Any]
) -> None:
    """
    Validates that all provided parameters match the signature of the model.

    Args:
        model_class: The model class to validate against
        provided_parameters: The parameters to validate
    """
    if type(model_class) == ClassDef:
        named_args, required_args = _infer_model_init_parameters_ast(model_class)
    else:
        named_args, required_args = _infer_model_init_parameters(model_class)

    # Check that there are no extra parameters
    if not named_args:
        return

    if provided_parameters and type(provided_parameters) is not dict:
        raise TypeError(
            f"Provided parameters must be a dict, not {type(provided_parameters)}"
        )

    for arg in provided_parameters:
        if arg not in named_args:
            raise ValueError(
                f"Provided parameter {arg} is not a valid init parameter for the model."
            )

    for arg in required_args:
        if arg not in provided_parameters:
            raise ValueError(
                f"Required init parameter {arg} was not provided for this model."
            )



from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List

from pydantic import BaseModel
from truss.patch.types import TrussSignature
from truss.templates.control.control.helpers.types import Patch


class ModelFrameworkType(Enum):
    SKLEARN = "sklearn"
    TENSORFLOW = "tensorflow"
    KERAS = "keras"
    PYTORCH = "pytorch"
    HUGGINGFACE_TRANSFORMER = "huggingface_transformer"
    XGBOOST = "xgboost"
    LIGHTGBM = "lightgbm"
    MLFLOW = "mlflow"
    CUSTOM = "custom"


@dataclass
class Example:
    name: str
    input: Any

    @staticmethod
    def from_dict(example_dict):
        return Example(
            name=example_dict["name"],
            input=example_dict["input"],
        )

    def to_dict(self) -> dict:
        return {
            "name": self.name,
            "input": self.input,
        }


@dataclass
class PatchDetails:
    prev_hash: str
    prev_signature: TrussSignature
    next_hash: str
    next_signature: TrussSignature
    patch_ops: List[Patch]

    def to_dict(self):
        return {
            "prev_hash": self.prev_hash,
            "prev_signature": self.prev_signature.to_dict(),
            "next_hash": self.next_hash,
            "next_signature": self.next_signature.to_dict(),
            "patch_ops": [patch_op.to_dict() for patch_op in self.patch_ops],
        }

    def is_empty(self) -> bool:
        # It's possible for prev_hash and next_hash to be different and yet
        # patch_ops to be empty, because certain parts of truss may be ignored.
        # e.g. training code is ignored for serving.
        return len(self.patch_ops) == 0

    @staticmethod
    def from_dict(patch_details: Dict) -> "PatchDetails":
        return PatchDetails(
            prev_hash=patch_details["prev_hash"],
            prev_signature=TrussSignature.from_dict(patch_details["prev_signature"]),
            next_hash=patch_details["next_hash"],
            next_signature=TrussSignature.from_dict(patch_details["next_signature"]),
            patch_ops=[
                Patch.from_dict(patch_op) for patch_op in patch_details["patch_ops"]
            ],
        )


class PatchRequest(BaseModel):
    """Request to patch running container"""

    hash: str
    prev_hash: str
    patches: List[Patch]

    def to_dict(self):
        return {
            "hash": self.hash,
            "prev_hash": self.prev_hash,
            "patches": [patch.to_dict() for patch in self.patches],
        }

    @staticmethod
    def from_dict(patch_request_dict: Dict):
        current_hash = patch_request_dict["hash"]
        prev_hash = patch_request_dict["prev_hash"]
        patches = patch_request_dict["patches"]
        return PatchRequest(hash=current_hash, prev_hash=prev_hash, patches=patches)



class Error(Exception):
    """Base Truss Error"""

    def __init__(self, message: str):
        super(Error, self).__init__(message)
        self.message = message
        self.type = type


class UnsupportedTypeError(Error):
    """Raised when a Pydantic field type is not supported"""

    pass


class MissingFieldError(Error):
    """Raised when a Pydantic field is missing"""

    pass


class MissingInputClassError(Error):
    """Raised when the user does not define an input class"""

    pass


class MissingOutputClassError(Error):
    """Raised when the user does not define an output class"""

    pass


class InvalidModelResponseError(Error):
    """Raised when the model returns an invalid response"""

    pass



import importlib
from pathlib import Path
from types import ModuleType
from typing import Any, List, Type, Union

import yaml
from jinja2 import Environment, FileSystemLoader, Template
from pydantic import BaseModel

# Copied from truss/templates/triton/model/utils/pydantic.py
FieldAnnotationType = Union[
    Type[int], Type[float], Type[str], Type[bool], Type[dict], List[Any]
]

# Mapping from pydantic types to Triton types
TYPE_MAPPING = {
    "str": "TYPE_STRING",
    "int": "TYPE_INT32",
    "float": "TYPE_FP32",
    "bool": "TYPE_BOOL",
    "dict": "TYPE_STRING",
}


class FieldDescriptor:
    """Descriptor for fields in a Pydantic model."""

    def __init__(self, name: str, triton_type: str, dims: List[int]):
        self.name = name
        self.triton_type = triton_type
        self.dims = dims

    @classmethod
    def from_pydantic_field(cls, field_name: str, field_info: Any) -> "FieldDescriptor":
        """Create a FieldDescriptor from a Pydantic field."""
        return cls(
            name=field_name,
            triton_type=get_triton_type(field_info.annotation),
            dims=get_triton_dims(field_info),
        )

    @classmethod
    def from_pydantic_class(
        cls, pydantic_class: Type[BaseModel]
    ) -> List["FieldDescriptor"]:
        """Create a list of FieldDescriptors from a Pydantic class."""
        return [
            cls.from_pydantic_field(field_name, field_info)
            for field_name, field_info in pydantic_class.__fields__.items()  # type: ignore
        ]

    def to_dict(self) -> dict:
        """Convert the FieldDescriptor to a dictionary."""
        return {
            "name": self.name,
            "dims": self.dims,
            "triton_type": self.triton_type,
        }


def read_template_from_fs(base_dir: Path, template_file_name: str) -> Template:
    """Read a Jinja template from the filesystem."""
    template_loader = FileSystemLoader(str(base_dir))
    template_env = Environment(loader=template_loader)
    return template_env.get_template(template_file_name)


def path_to_module(path: Path) -> str:
    """Convert a filesystem path to a Python module string."""
    return str(path).replace("/", ".")


def is_conlist(pydantic_field_annotation: FieldAnnotationType) -> bool:
    """Check if a Pydantic field annotation is a conlist."""
    return (
        hasattr(pydantic_field_annotation, "__origin__")
        and pydantic_field_annotation.__origin__ == list
    )


def get_triton_type(pydantic_field: FieldAnnotationType) -> str:
    """Get the corresponding Triton type for a Pydantic field."""
    if is_conlist(pydantic_field):
        pydantic_field = pydantic_field.__args__[0]  # type: ignore

    type_name = (
        pydantic_field.__name__
        if hasattr(pydantic_field, "__name__")
        else str(pydantic_field)
    )

    if type_name not in TYPE_MAPPING:
        raise TypeError(f"Unsupported type: {type_name}")

    return TYPE_MAPPING[type_name]


def get_triton_dims(pydantic_type: Type) -> List[int]:
    """Get the dimensions for a Pydantic type."""
    if is_conlist(pydantic_type.annotation):
        dims = next(
            (m for m in pydantic_type.metadata if hasattr(m, "min_length")), None
        )
        if dims and dims.min_length == dims.max_length:
            return [-1, dims.min_length]
        else:
            return [-1, -1]
    return [-1]


def validate_user_model_class(module: ModuleType, required_attributes: List[str]):
    """
    Validates that a user model class has the required attributes.

    Args:
        module (ModuleType): The module containing the user model class.
        required_attributes (list): A list of required attributes.

    Raises:
        AttributeError: If a required attribute is missing.
    """
    for attribute in required_attributes:
        if not hasattr(module, attribute):
            raise AttributeError(
                f"Truss model class is missing {attribute} attribute. For Triton, "
                f"we require a Pydantic model class that corresponds to your model "
                f"input and model output."
            )


def generate_config_pbtxt(
    input_class: Type[BaseModel],
    output_class: Type[BaseModel],
    template_path: Path,
    template_name: str = "config.pbtxt.jinja",
    max_batch_size: int = 1,
    num_replicas: int = 1,
    dynamic_batch_delay_ms: int = 0,
    is_gpu: bool = False,
) -> str:
    """Generate the config.pbtxt file content."""
    config_params = {
        "max_batch_size": max_batch_size,
        "num_replicas": num_replicas,
        "is_gpu": is_gpu,
    }
    template = read_template_from_fs(template_path, template_name)
    input_cls_field_descriptors = FieldDescriptor.from_pydantic_class(input_class)
    output_cls_field_descriptors = FieldDescriptor.from_pydantic_class(output_class)
    inputs = [field.to_dict() for field in input_cls_field_descriptors]
    outputs = [field.to_dict() for field in output_cls_field_descriptors]
    if dynamic_batch_delay_ms > 0:
        config_params["dynamic_batching_delay_microseconds"] = dynamic_batch_delay_ms
    return template.render(inputs=inputs, outputs=outputs, **config_params)


def load_config(user_truss_path: Path) -> dict:
    """Load the configuration from the YAML file."""
    with open(user_truss_path / "config.yaml", "r") as f:
        return yaml.safe_load(f)


def extract_config_arguments(config: dict) -> dict:
    """Extract configuration arguments from the loaded config."""
    build = config.get("build", {})
    build.pop("model_server", None)
    return {k: v for d in build["arguments"] for k, v in d.items()}


def determine_gpu_usage(config: dict) -> bool:
    """Determine if GPU is enabled based on the config."""
    return config.get("resources", {}).get("use_gpu", False)


def import_user_model(user_truss_path: Path, model_class_filename: str) -> ModuleType:
    """Import the user model class."""
    model_module_name = str(Path(model_class_filename).with_suffix(""))
    return importlib.import_module(
        f"{path_to_module(user_truss_path)}.model.{model_module_name}"
    )


def write_config_to_file(model_repository_path: Path, config_pbtxt: str):
    """Write the generated config.pbtxt to the model directory."""
    with open(model_repository_path / "config.pbtxt", "w") as f:
        f.write(config_pbtxt)


def main():
    """Main function to generate the config.pbtxt."""
    model_repository_path = Path("model")
    user_truss_path = model_repository_path / "1" / "truss"

    # Load and process configuration
    config = load_config(user_truss_path)
    config_arguments = extract_config_arguments(config)
    config_arguments["is_gpu"] = determine_gpu_usage(config)

    # Import and validate user model
    module = import_user_model(user_truss_path, config["model_class_filename"])
    input_cls_name = config.get("input_type_name", "Input")
    output_cls_name = config.get("output_type_name", "Output")
    validate_user_model_class(module, [input_cls_name, output_cls_name])

    # Generate and write config.pbtxt
    input_cls = getattr(module, input_cls_name)
    output_cls = getattr(module, output_cls_name)
    triton_config = generate_config_pbtxt(
        input_cls, output_cls, Path("/app"), **config_arguments
    )
    write_config_to_file(model_repository_path, triton_config)


if __name__ == "__main__":
    main()



import inspect


def _signature_accepts_kwargs(signature: inspect.Signature) -> bool:
    """Checks if a signature accepts **kwargs."""
    for param in signature.parameters.values():
        if param.kind == inspect.Parameter.VAR_KEYWORD:
            return True
    return False


def signature_accepts_keyword_arg(signature: inspect.Signature, kwarg: str) -> bool:
    """Checks if a signature accepts a keyword argument with the given name."""
    return kwarg in signature.parameters or _signature_accepts_kwargs(signature)



from pathlib import Path
from typing import Dict, List, Optional

from truss.patch.hash import file_content_hash_str
from truss.patch.utils import path_matches_any_pattern


def directory_content_signature(
    root: Path, ignore_patterns: Optional[List[str]] = None
) -> Dict[str, str]:
    """Calculate content signature of a filesystem directory.

    Sort all files by path, store file path with content hash.
    Signatures are meant to track changes in directory. e.g.
    A previous signature and the directory can be combined to create
    a patch from the previous state.

    Hash of directories is marked None.
    """

    paths = [
        path
        for path in root.glob("**/*")
        if not path_matches_any_pattern(path.relative_to(root), ignore_patterns)
    ]
    paths.sort(key=lambda p: p.relative_to(root))

    def path_hash(pth: Path):
        if pth.is_file():
            return file_content_hash_str(pth)

    return {str(path.relative_to(root)): path_hash(path) for path in paths}



import json
from typing import List, Type

import numpy as np
from pydantic import BaseModel
from utils.errors import InvalidModelResponseError
from utils.pydantic import PYTHON_TYPE_TO_NP_DTYPE, inspect_pydantic_model
from utils.triton import (
    InferenceRequest,
    InferenceResponse,
    convert_tensor_to_python_type,
    create_inference_response,
    create_tensor,
    get_input_tensor_by_name,
)


def transform_triton_to_pydantic(
    triton_requests: List[InferenceRequest],
    pydantic_type: Type[BaseModel],
) -> List[BaseModel]:
    """
    Transforms a list of Triton requests into a list of Pydantic objects.

    This function inspects the fields of the given Pydantic type and extracts
    the corresponding tensors from the Triton requests. It then converts these
    tensors into Python types and uses them to instantiate the Pydantic objects.

    Args:
        triton_requests (list): A list of Triton request objects.
        pydantic_type (Type[BaseModel]): The Pydantic model type to which the Triton requests should be transformed.

    Returns:
        list: A list of Pydantic objects instantiated from the Triton requests.

    Raises:
        ValueError: If a tensor corresponding to a Pydantic field cannot be found in the Triton request.
        TypeError: If a tensor corresponding to a Pydantic field has an unsupported type.
    """
    fields = inspect_pydantic_model(pydantic_type)
    results = []

    for request in triton_requests:
        data = {}
        for field in fields:
            field_name, field_type = field.values()
            tensor = get_input_tensor_by_name(request, field_name)
            data[field_name] = convert_tensor_to_python_type(tensor, field_type)
        results.append(pydantic_type(**data))

    return results


def transform_pydantic_to_triton(
    pydantic_objects: List[BaseModel],
) -> List[InferenceResponse]:
    """
    Transforms a list of Pydantic objects into a list of Triton inference responses.

    This function iterates over the fields of each Pydantic object, determines the
    appropriate tensor data type, and creates a tensor for each field. These tensors
    are then used to create Triton inference responses.

    Args:
        pydantic_objects (list): A list of Pydantic objects to be transformed.

    Returns:
        list: A list of Triton inference responses created from the Pydantic objects.

    Raises:
        ValueError: If a Python type in the Pydantic object does not have a corresponding numpy dtype.
    """
    results = []

    if not isinstance(pydantic_objects, list):
        raise InvalidModelResponseError(
            "Truss did not return a list. Please ensure that your model returns a list \
            of Pydantic objects even if the batch size is 1."
        )

    if not all(isinstance(obj, BaseModel) for obj in pydantic_objects):
        raise InvalidModelResponseError(
            "Truss returned a list of objects that are not Pydantic models. Please \
            ensure that your model returns a list of Pydantic objects even if the \
            batch size is 1."
        )

    for obj in pydantic_objects:
        tensors = []
        for field, value in obj.dict().items():
            if isinstance(value, list):
                dtype = PYTHON_TYPE_TO_NP_DTYPE[type(value[0])]
            elif isinstance(value, dict):
                dtype = PYTHON_TYPE_TO_NP_DTYPE[type(value)]
                value = [json.dumps(value)]
            else:
                dtype = PYTHON_TYPE_TO_NP_DTYPE[type(value)]
                value = [value]
            tensors.append(create_tensor(field, np.array(value, dtype=dtype)))
        results.append(create_inference_response(tensors))

    return results



from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Dict, Optional

import requests
from truss.truss_handle import TrussHandle


class TrussService(ABC):
    """
    Define the abstract base class for a TrussService.

    A TrussService interacts with a service at a given URL and can either be in draft or non-dreaf mode.

    Attributes:
        _service_url: The URL of the service.
        _is_draft: A boolean indicating if the service is in draft mode.

    Args:
        service_url: The URL of the service.
        is_draft: A boolean indicating if the service is in draft mode.
        **kwargs: Additional keyword arguments to initialize the service.

    """

    def __init__(self, service_url: str, is_draft: bool, **kwargs):
        self._service_url = service_url
        self._is_draft = is_draft

    def _send_request(
        self,
        url: str,
        method: str,
        headers: Optional[Dict] = None,
        data: Optional[Dict] = None,
        stream: Optional[bool] = False,
    ):
        """
        Send a HTTP request.

        Args:
            url: The URL to send the request to.
            method: The HTTP method to use.
            headers: Optional dictionary of headers to include in the request.
            data: Optional dictionary of data to include in the request.

        Raises:
            ValueError: If an unsupported method is used or a POST request has no data.

        Returns:
            A Response object resulting from the request.

        """
        if not headers:
            headers = {}

        auth_header = self.authenticate()
        headers = {**headers, **auth_header}
        if method == "GET":
            response = requests.request(method, url, headers=headers, stream=stream)
        elif method == "POST":
            if data is None:
                raise ValueError("POST request must have data")
            response = requests.request(
                method, url, json=data, headers=headers, stream=stream
            )
        else:
            raise ValueError(f"Unsupported method: {method}")

        if response.status_code == 401:
            raise ValueError(
                f"Authentication failed with status code {response.status_code}"
            )

        return response

    @property
    def is_draft(self):
        """
        Check if the service is in draft mode.

        Returns:
            A boolean indicating if the service is in draft mode.
        """
        return self._is_draft

    @abstractmethod
    def is_live(self) -> bool:
        """
        Check if the service is live.

        Sends a GET request to the root of the service and returns whether it is successful.

        Returns:
            A boolean indicating if the service is live.
        """
        liveness_url = f"{self._service_url}/"
        response = self._send_request(liveness_url, "GET", {})
        return response.status_code == 200

    @abstractmethod
    def is_ready(self) -> bool:
        """
        Check if the service is ready.

        Sends a GET request to the model path of the service and returns whether it is successful.

        Returns:
            A boolean indicating if the service is ready.
        """
        readiness_url = f"{self._service_url}/v1/models/model"
        response = self._send_request(readiness_url, "GET", {})
        return response.status_code == 200

    def predict(self, model_request_body: Dict):
        """
        Send a prediction request to the service.

        Args:
            model_request_body: A dictionary representing the body of the prediction request.

        Returns:
            A Response object resulting from the prediction request.
        """
        invocation_url = f"{self._service_url}/v1/models/model:predict"
        response = self._send_request(invocation_url, "POST", data=model_request_body)
        return response

    def patch(self):
        """
        Patch the service. TrussServices in draft mode can be patched.
        """
        raise NotImplementedError

    @abstractmethod
    def authenticate(self) -> dict:
        """
        Authenticate to the service.

        This method should be implemented in subclasses and return a dictionary of headers
        to include in requests to the service with authentication information.
        """
        return {}

    @abstractmethod
    def logs_url(self, base_url: str) -> str:
        """
        Get the URL for the service logs.
        """
        pass


class TrussRemote(ABC):
    """
    Define the abstract base class for a remote Truss service.

    A remote Truss service is a service that can push a TrussHandle to a remote location.
    The `push` and `authenticate` methods should be implemented in subclasses.

    Attributes:
        _remote_url: The URL of the remote service.

    Args:
        remote_url: The URL of the remote service.
        **kwargs: Additional keyword arguments needed to initialize the remote service.

    """

    def __init__(self, remote_url: str, **kwargs):
        self._remote_url = remote_url

    @abstractmethod
    def push(self, truss_handle: TrussHandle, **kwargs):
        """
        Push a TrussHandle to the remote service.

        This method should be implemented in subclasses and return a TrussService
        object for interacting with the remote service.

        Args:
            truss_handle: The TrussHandle to push to the remote service.
            **kwargs: Additional keyword arguments for the push operation.

        """
        pass

    @abstractmethod
    def authenticate(self, **kwargs):
        """
        Authenticate the user to push to the remote service.

        This method should be implemented in subclasses. It should check whether
        the user has valid authentication credentials to push to the remote service.
        If not, it should raise an exception.

        Args:
            **kwargs: Additional keyword arguments for the authentication operation.
        """
        pass

    @abstractmethod
    def get_service(self, **kwargs):
        """
        Get a TrussService object for interacting with the remote service.

        This method should be implemented in subclasses and return a TrussService
        object for interacting with the remote service. This support keyword args,
        so that implementing remotes can be flexible with how a service is instantiated.

        Args:
            **kwargs: Keyword arguments for the get_service operation.

        """
        pass

    @abstractmethod
    def get_remote_logs_url(self, service: TrussService) -> str:
        """
        Get the URL for the remote service logs.

        Args:
            service: The TrussService object for interacting with the remote service.
        """
        pass

    @abstractmethod
    def sync_truss_to_dev_version_by_name(self, model_name: str, target_directory: str):
        """
        This method watches for changes to files in the `target_directory`,
        and syncs them to the development version of the model, identified
        by name.

        Args:
            model_name: The name of the model to sync to the dev version
            target_directory: The directory to sync the model to
        """

        pass


@dataclass
class RemoteConfig:
    """Class to hold configs for various remotes"""

    name: str
    configs: Dict = field(default_factory=dict)



from truss.contexts.local_loader.utils import prepare_secrets
from truss.local.local_config_handler import LocalConfigHandler
from truss.truss_handle import TrussHandle
from truss.truss_spec import TrussSpec


def test_prepare_secrets(custom_model_truss_dir, tmp_path):
    orig_truss_config_dir = LocalConfigHandler.TRUSS_CONFIG_DIR
    LocalConfigHandler.TRUSS_CONFIG_DIR = tmp_path
    try:
        LocalConfigHandler.set_secret("secret_name", "secret_value")
        handle = TrussHandle(custom_model_truss_dir)
        handle.add_secret("secret_name")
        spec = TrussSpec(custom_model_truss_dir)
        secrets = prepare_secrets(spec)
        assert secrets["secret_name"] == "secret_value"
    finally:
        LocalConfigHandler.TRUSS_CONFIG_DIR = orig_truss_config_dir



from unittest import mock

import pytest
import requests
from requests import Response
from truss.remote.baseten.api import BasetenApi
from truss.remote.baseten.error import ApiError


def mock_auth_service():
    auth_service = mock.Mock()
    auth_token = mock.Mock(headers=lambda: {"Authorization": "Api-Key token"})
    auth_service.authenticate.return_value = auth_token
    return auth_service


def mock_successful_response():
    response = Response()
    response.status_code = 200
    response.json = mock.Mock(return_value={"data": {"status": "success"}})
    return response


def mock_graphql_error_response():
    response = Response()
    response.status_code = 200
    response.json = mock.Mock(return_value={"errors": [{"message": "error"}]})
    return response


def mock_unsuccessful_response():
    response = Response()
    response.status_code = 400
    return response


@mock.patch("truss.remote.baseten.auth.AuthService")
@mock.patch("requests.post", return_value=mock_successful_response())
def test_post_graphql_query_success(mock_post, mock_auth_service):
    api_url = "https://test.com/api"
    api = BasetenApi(api_url, mock_auth_service)

    response_data = {"data": {"status": "success"}}

    result = api._post_graphql_query("sample_query_string")

    assert result == response_data


@mock.patch("truss.remote.baseten.auth.AuthService")
@mock.patch("requests.post", return_value=mock_graphql_error_response())
def test_post_graphql_query_error(mock_post, mock_auth_service):
    api_url = "https://test.com/api"
    api = BasetenApi(api_url, mock_auth_service)

    with pytest.raises(ApiError):
        api._post_graphql_query("sample_query_string")


@mock.patch("truss.remote.baseten.auth.AuthService")
@mock.patch("requests.post", return_value=mock_unsuccessful_response())
def test_post_requests_error(mock_post, mock_auth_service):
    api_url = "https://test.com/api"
    api = BasetenApi(api_url, mock_auth_service)
    with pytest.raises(requests.exceptions.HTTPError):
        api._post_graphql_query("sample_query_string")
